#include "StdInc.h"
#include "Misc.h"

CharPtr StrLToA (CharPtr lpStr, Long lNum)
{
    Boolean bNeg=false;
    CharPtr ptr1, ptr2;
    Char ch;

    if (lNum < 0)
    {
        bNeg = true;
        lNum *= -1;
    }

    ptr1 = lpStr;
    ptr2 = lpStr;
    do
    {
        *ptr1++ = (lNum % 10) + '0';
        lNum /= 10;
    } while (lNum);
    if (bNeg)
        *ptr1++ = '-';
    *ptr1-- = '\0';
    while (ptr1 > ptr2)
    {
        ch = *ptr1;
        *ptr1-- = *ptr2;
        *ptr2++ = ch;
    }

    return lpStr;
}

Long StrAToL (CharPtr lpStr)
{
    Long lResult = 0;
    Boolean bNeg = false;

    if (*lpStr == '+')
        lpStr++;
    else if (*lpStr == '-')
    {
        lpStr++;
        bNeg = true;
    }
    while (*lpStr)
    {
        lResult *= 10;
        lResult += *lpStr++ - '0';
    }
    if (bNeg)
        lResult *= -1;

    return lResult;
}

void QShuffle (BytePtr base, Int numOfElements, Int width)
{
	BytePtr ptr1, ptr2;
    Byte ch;
	Int i, pos;

	while (numOfElements)
	{
		pos = SysRandom(0) % numOfElements--;
		if (pos == numOfElements) continue;
		ptr1 = (BytePtr)(base + pos * width);
		ptr2 = (BytePtr)(base + numOfElements * width);
		i = width;
		while (i--)
		{
			ch = *ptr1;
			*ptr1++ = *ptr2;
			*ptr2++ = ch;
		}
	}
}

void InitPopup (FormPtr fpForm, Word wTriggerId, Word wListId, Int iCurrent)
{
    CharPtr cpPtr;

    cpPtr = LstGetSelectionText(FrmGetObjectPtr(fpForm, 
        FrmGetObjectIndex(fpForm, wListId)), iCurrent);
    if (!cpPtr)
        cpPtr = "";
    FrmCopyLabel(fpForm, wTriggerId, cpPtr);
}

Int SelectPopup (FormPtr fpForm, Word wTriggerId, Word wListId, Int iCurrent)
{
    ListPtr lpPopup = FrmGetObjectPtr(fpForm, FrmGetObjectIndex(fpForm, wListId));
    Word iSelect;

    LstSetSelection(lpPopup, iCurrent);
    LstMakeItemVisible(lpPopup, iCurrent);
    LstPopupList(lpPopup);
    iSelect = LstGetSelection( lpPopup );
    if( iSelect != noListSelection )
    {
        iCurrent = iSelect;
        FrmHideObject(fpForm, FrmGetObjectIndex(fpForm, wTriggerId));
        FrmCopyLabel(fpForm, wTriggerId, LstGetSelectionText(lpPopup, iCurrent));
        FrmShowObject(fpForm, FrmGetObjectIndex(fpForm, wTriggerId));
    }

    return iCurrent;
}

Int ChangePopup (FormPtr fpForm, Word wTriggerId, Word wListId, Int iCurrent)
{
    ListPtr lpPopup = FrmGetObjectPtr(fpForm, FrmGetObjectIndex(fpForm, wListId));
    Word iSelect;

    LstSetSelection(lpPopup, iCurrent);
    iSelect = LstGetSelection( lpPopup );
    if( iSelect != noListSelection )
    {
        iCurrent = iSelect;
        FrmHideObject(fpForm, FrmGetObjectIndex(fpForm, wTriggerId));
        FrmCopyLabel(fpForm, wTriggerId, LstGetSelectionText(lpPopup, iCurrent));
        FrmShowObject(fpForm, FrmGetObjectIndex(fpForm, wTriggerId));
    }

    return iCurrent;
}

Int InitDynamicPopup (FormPtr fpForm, Word wTriggerId)
{
	CtlSetLabel(FrmGetObjectPtr(fpForm, FrmGetObjectIndex(fpForm, wTriggerId)), 0);

	return 0;
}

Int SetDynamicPopup (FormPtr fpForm, Word wTriggerId, CharPtr pText)
{
	ControlPtr lpTrigger = FrmGetObjectPtr(fpForm, FrmGetObjectIndex(fpForm, wTriggerId));
	CharPtr pTemp;

	pTemp = CtlGetLabel(lpTrigger);
	if (pTemp) MemPtrFree(pTemp);
	pTemp = MemPtrNew(StrLen(pText) + 1);
	StrCopy(pTemp, pText);
	CtlSetLabel(lpTrigger, pTemp);

	return 0;
}

Int SelectDynamicPopup (FormPtr fpForm, Word wTriggerId, Word wListId, Int iCurrent, Char **pList, Int iSize)
{
	ControlPtr lpTrigger = FrmGetObjectPtr(fpForm, FrmGetObjectIndex(fpForm, wTriggerId));
    ListPtr lpPopup = FrmGetObjectPtr(fpForm, FrmGetObjectIndex(fpForm, wListId));
    Int iSelect, i, iMaxSize;
	CharPtr pTemp, pOld;

	iMaxSize = 0;
	for (i=0; i<iSize; i++)
	{
		if (StrLen(pList[i]) > iMaxSize)
			iMaxSize = StrLen(pList[i]);
	}

	pOld = CtlGetLabel(lpTrigger);
	pTemp = MemPtrNew(iMaxSize + 1);
	// Don't need the old now.
	if (pOld)
	{	
		StrCopy(pTemp, pOld);
		MemPtrFree(pOld);
	}
	else
	{
		*pTemp = '\0';
	}

	LstSetHeight(lpPopup, (iSize > 10) ? 10 : iSize);
	LstSetListChoices(lpPopup, pList, iSize);

    LstSetSelection(lpPopup, iCurrent);
    LstMakeItemVisible(lpPopup, iCurrent);
    LstPopupList(lpPopup);
    iSelect = LstGetSelection( lpPopup );
    if( iSelect != noListSelection )
    {
        iCurrent = iSelect;
		StrCopy(pTemp, LstGetSelectionText(lpPopup, iCurrent));
    }
	CtlSetLabel(lpTrigger, pTemp);

    return iCurrent;
}

VoidHand SetField (FormPtr fpForm, Word wFieldId, ULong ulMaxChars, CharPtr pText)
{
    FieldPtr fpField;
    VoidHand hNewHandle, hOldHandle;
	CharPtr pNewText, pOldText;
	ULong ulOldSize;
	Boolean bUseOld = false;

    fpField = FrmGetObjectPtr(fpForm, FrmGetObjectIndex(fpForm, wFieldId));
    hNewHandle = MemHandleNew(ulMaxChars+1);
    if (hNewHandle)
    {
		pNewText = MemHandleLock(hNewHandle);
		hOldHandle = (VoidHand)FldGetTextHandle(fpField);
        if (pText)
        {
			pOldText = pText;
        }
        else if (hOldHandle)
		{
			pOldText = MemHandleLock(hOldHandle);
			bUseOld = true;
		}
		else
		{
			pOldText = "";
		}

		ulOldSize = StrLen(pOldText);
		if (ulOldSize > ulMaxChars)
			ulOldSize = ulMaxChars;

		MemMove(pNewText, pOldText, ulOldSize);
		pNewText [ulOldSize] = '\0';

		MemHandleUnlock(hNewHandle);
        FldSetMaxChars(fpField, ulMaxChars);
        FldSetTextHandle(fpField, (Handle)hNewHandle);

        if (hOldHandle)
		{
			if (bUseOld)
				MemHandleUnlock(hOldHandle);
            MemHandleFree(hOldHandle);
		}
    }

    return hNewHandle;
}

void DrawCenteredChars (CharPtr chars, Word len, RectanglePtr r)
{
    SWord x, y, w, h;

    w = FntCharsWidth(chars, len);
    h = FntLineHeight();

    x = r->topLeft.x + ((r->extent.x - w) / 2);
    y = r->topLeft.y + ((r->extent.y - h) / 2);

//    WinEraseChars(chars, len, x, y);
    WinDrawChars(chars, len, x, y);
}

void SetCheck (FormPtr fpForm, Word wFieldId, Boolean bCheckState)
{
	CtlSetValue(FrmGetObjectPtr(fpForm, FrmGetObjectIndex(fpForm, wFieldId)), (short)bCheckState);
}

Boolean GetCheck (FormPtr fpForm, Word wFieldId)
{
	return (Boolean)CtlGetValue(FrmGetObjectPtr(fpForm, FrmGetObjectIndex(fpForm, wFieldId)));
}

void SetPushButton (FormPtr fpForm, Word wStartFieldId, Word wEndFieldId, Word wFieldId)
{
	Word w;
	Boolean bCheck, bChecked;

	for (w=wStartFieldId; w<=wEndFieldId; w++)
	{
		if ((w != wFieldId) && GetCheck(fpForm, w))
		{
			SetCheck(fpForm, w, false);
		}
	}
	SetCheck(fpForm, wFieldId, true);
}

Word GetPushButton (FormPtr fpForm, Word wStartFieldId, Word wEndFieldId)
{
	Word w;

	for (w=wStartFieldId; w<=wEndFieldId; w++)
	{
		if (GetCheck(fpForm, w))
		{
			return w;
		}
	}

	return 0;
}

void ReadRecord (DmOpenRef hDataBase, UInt uiIndex, VoidPtr pVoid, ULong ulLength)
{
    VoidHand hRecord;
    Ptr pRecord;

	if (hDataBase)
	{
		if (DmNumRecords(hDataBase) > uiIndex)
		{
			hRecord = DmGetRecord(hDataBase, uiIndex);
			// If the record is smaller read in a partial record.
			if (ulLength > MemHandleSize(hRecord))
				ulLength = MemHandleSize(hRecord);
			pRecord = MemHandleLock(hRecord);
			MemMove(pVoid, pRecord, ulLength);
			MemHandleUnlock(hRecord); 
			DmReleaseRecord(hDataBase, uiIndex, false);  
		}
	}
}

void OverwriteRecord (DmOpenRef hDataBase, UInt uiIndex, VoidPtr pVoid, ULong ulLength)
{
    VoidHand hRecord;
    Ptr pRecord;

	if (hDataBase)
	{
		// Does a record exist?
		if (DmNumRecords(hDataBase) > uiIndex)
		{
			hRecord = DmGetRecord(hDataBase, uiIndex);
			if (ulLength == MemHandleSize(hRecord))
			{
				// The same size record, overwrite it.
				pRecord = MemHandleLock(hRecord);
				DmWrite(pRecord, 0, pVoid, ulLength);
				MemHandleUnlock(hRecord); 
				DmReleaseRecord(hDataBase, uiIndex, false);
				return;
			}
			else
			{
				// New record erase it.
				DmReleaseRecord(hDataBase, uiIndex, false);
				DmRemoveRecord(hDataBase, uiIndex);
			}
		}

		// Write a new record.
		InsertRecord(hDataBase, uiIndex, pVoid, ulLength);
	}
}

void InsertRecord (DmOpenRef hDataBase, UInt uiIndex, VoidPtr pVoid, ULong ulLength)
{
    VoidHand hRecord;
    Ptr pRecord;

	if (hDataBase)
	{
		// Write the new record.
		hRecord = DmNewRecord(hDataBase, &uiIndex, ulLength);
		pRecord = MemHandleLock(hRecord);
		DmWrite(pRecord, 0, pVoid, ulLength);
		MemHandleUnlock(hRecord); 
		DmReleaseRecord(hDataBase, uiIndex, true);  
	}
}
