#include "StdInc.h"
#include "RollEmUpd.h"
#include "About.h"
#include "Sound.h"

Int StartApplication(void);
void StopApplication(void);
Boolean OpenDatabase(void);
Boolean CreateDatabase(void);
Boolean CloseDatabase(void);

void EventLoop(void);
Boolean MainProc(EventPtr event);

void InitializePage (DieRoll101Ptr pDieRoll101);
void ReadPage (Word wPage, DieRoll101Ptr pDieRoll101);
void WritePage (Word wPage, DieRoll101Ptr pDieRoll101);

void CreateTitle (DieRoll101Ptr pDieRoll101, CharPtr charPtr, Word wLen);
Boolean CheckValue (Byte iCompareType, Long lCompareValue, Long lResult);
void PlayBeep (Byte iBeepType);

void ChangeCurrent (Word wId);
void CustomizeButton (Word wId);
void CheckCompare (DieRoll101 dr, Long lResult);

Boolean Initialize ();
Boolean Deinitialize ();

void AddPage ();
void DeletePage ();
void ClearPage ();

Boolean ConvertDatabase (void);
Boolean CreateDatabase101 (void);
Boolean InitializeDatabase101 (void);
Boolean ClearDatabase (void);
Boolean ConvertDatabase101to110 (void);

void ReadRecord (DmOpenRef hRollEmDB, UInt uiIndex, VoidPtr pVoid, ULong ulLength);
void OverwriteRecord (DmOpenRef hRollEmDB, UInt uiIndex, VoidPtr pVoid, ULong ulLength);
void InsertRecord (DmOpenRef hRollEmDB, UInt uiIndex, VoidPtr pVoid, ULong ulLength);

void DrawInfo ();

DmOpenRef g_hRollEmDB;
Char szPrefsDatabaseName[] = "Prefs-RlEm";

Word g_wVersion;

DWord  PilotMain (Word cmd, Ptr cmdPBP, Word launchFlags)
{
    Int error;

    if (cmd == sysAppLaunchCmdNormalLaunch)
    {
        error = StartApplication();	// Application start code
        if (error) return error;

        EventLoop();	// Event loop

        StopApplication ();	// Application stop code
    }
    return 0;
}

Int StartApplication(void)
{
	Int error;

	error = Initialize();
	if (error) 
		return error;
	FrmGotoForm(IDF_MAIN);
}

void StopApplication(void)
{
    Deinitialize();
}

Boolean OpenDatabase(void)
{
    VoidHand hRecord;
    Ptr pRecord;
    UInt uiIndex;
    ULong ulLength;
    Int i;

    g_hRollEmDB = DmOpenDatabaseByTypeCreator(ROLLEM_DBTYPE, ROLLEM_APPID, dmModeReadWrite);
    if (g_hRollEmDB) 
    {
		ReadRecord(g_hRollEmDB, PREFS_RECORD, &g_wVersion, sizeof(Word));
    }

	return 0;
}

Boolean ConvertDatabase (void)
{
//	OpenDatabase();
	if (g_wVersion < 101)
	{
		CreateDatabase101();
	}
	if (g_wVersion < 110)
	{
		ConvertDatabase101to110();
	}
//	CloseDatabase();
	DrawInfo();

	return 0;
}

Boolean CreateDatabase101 (void)
{
	if (!g_hRollEmDB)
	{
		if (DmCreateDatabase(0, szPrefsDatabaseName, ROLLEM_APPID, ROLLEM_DBTYPE, false)) 
			return 1;
		g_hRollEmDB = DmOpenDatabaseByTypeCreator(ROLLEM_DBTYPE, ROLLEM_APPID, dmModeReadWrite);
	}

	return InitializeDatabase101();
}

Boolean CloseDatabase()
{
	if (g_hRollEmDB)
	{
		// DmResetRecordStates(g_hRollEmDB); // This crashes the app.
		DmCloseDatabase(g_hRollEmDB);
	}

    return 0;
}

Boolean ClearDatabase ()
{
	if (g_hRollEmDB)
	{
		UInt uiRecords;

		// Delete all records.
		uiRecords = DmNumRecords(g_hRollEmDB);
		while (uiRecords--)
			DmRemoveRecord(g_hRollEmDB, 0);
	}

    return 0;
}

Boolean InitializeDatabase101 (void)
{
    UInt uiIndex;
    ULong ulLength;
    Int i;
	DieRoll101Ptr pDieRoll;
	Prefs101 vPrefs101;

	// This is the base database make a clean state.
	ClearDatabase();

	// Create the prefs record.
	// Initialize the prefs
	MemSet(&vPrefs101, sizeof(Prefs101), '\0');
	vPrefs101.wVersion = 101;
	vPrefs101.wTotalPages = DEFAULT_PAGES;

	// Write the prefs out to the database
	InsertRecord(g_hRollEmDB, PREFS_RECORD, &vPrefs101, sizeof(Prefs101));

	// Create the page records
    pDieRoll = MemPtrNew(sizeof(DieRoll101) * NUM_BUTTONS);
	if (pDieRoll)
	{
		// Initialize the page
	    InitializePage(pDieRoll);
		for (i=0; i<DEFAULT_PAGES; i++)
		{
			// Write the prefs out to the database
			InsertRecord(g_hRollEmDB, PAGE2REC101(i), pDieRoll, (sizeof(DieRoll101) * NUM_BUTTONS));
		}
	}

	// We now have a 101 database set the version version number
	g_wVersion = 101;

    return 0;
}

void EventLoop(void)
{
    short err;
    Int formID;
    FormPtr fpForm;
    EventType event;

    do
    {
        EvtGetEvent(&event, -1);

        if (SysHandleEvent(&event)) continue;
        if (MenuHandleEvent((void *)0, &event, &err)) continue;

        if (event.eType == frmLoadEvent)
        {
            formID = event.data.frmLoad.formID;
            fpForm = FrmInitForm(formID);
            FrmSetActiveForm(fpForm);
            switch (formID) 
            {
				// Don't need to put form handlers for FrmDoDialog.
                case IDF_MAIN:
                    FrmSetEventHandler(fpForm, (FormEventHandlerPtr) MainProc);
                    break;
                case IDF_SOUND:
                    FrmSetEventHandler(fpForm, (FormEventHandlerPtr) SoundProc);
                    break;
                case IDF_ABOUT:
                    FrmSetEventHandler(fpForm, (FormEventHandlerPtr) AboutProc);
                    break;
            }
        }
        FrmDispatchEvent(&event);
    } while(event.eType != appStopEvent);
}

Boolean MainProc(EventPtr event)
{
    Int       handled = 0;

    switch (event->eType) 
    {
        case frmOpenEvent:
        {
            FormPtr fpForm;
            fpForm = FrmGetActiveForm();
            FrmDrawForm(fpForm);
			DrawInfo();
            handled = 1;
            break;
        }
/*    
		case ctlSelectEvent:  // A control button was pressed and released.
			switch (event->data.ctlEnter.controlID)
			{
			}
			handled = 1;
			break;
*/
        case menuEvent:
			switch (event->data.menu.itemID)
			{
			case IDM_UPDATE_DATABASE:
				ConvertDatabase();
				break;
			case IDM_UPDATE_SOUND:
			{
				Word wVersion = 0;

				ReadRecord(g_hRollEmDB, PREFS_RECORD, &wVersion, sizeof(Word));
				if (wVersion == VERSION_NUMBER)
				{
					FrmPopupForm(IDF_SOUND);
				}
				else
				{
					FrmCustomAlert(IDA_OUT_OF_DATE, NULL, NULL, NULL);
				}
				break;
			}
			case IDM_OPTIONS_ABOUT:
				FrmPopupForm(IDF_ABOUT);
				break;
			}
			handled = 1;
			break;
    }
    return handled;
}

void InitializePage (DieRoll101Ptr pDieRoll101)
{
    Int iDefaultButtons [NUM_BUTTONS] = { 2, 4, 6, 8, 10, 12, 20, 100 };
    Int i;

    MemSet(pDieRoll101, (sizeof(DieRoll101) * NUM_BUTTONS), '\0');
    for(i=IDC_BUTTON_FIRST; i<=IDC_BUTTON_LAST; i++)
    {
        pDieRoll101->iDie = iDefaultButtons [ID2NUM(i)];
        pDieRoll101->iNumber = 1;
        pDieRoll101->iModifier = 0;

        CreateTitle(pDieRoll101, pDieRoll101->szTitle, MAX_TITLE_LEN);

        pDieRoll101++;
    }
}

Boolean Initialize ()
{
	g_wVersion = 0;
	OpenDatabase();

    return 0;
}

Boolean Deinitialize ()
{
	CloseDatabase();

    return 0;
}

void CreateTitle (DieRoll101Ptr pDieRoll101, CharPtr charPtr, Word wLen)
{
	Char szTemp[MAX_TEMPSTRING_LEN];

	*szTemp = '\0';
    if ((pDieRoll101->iBest) && (pDieRoll101->iBest < pDieRoll101->iNumber))
    {
        StrCat(szTemp, "Best ");
        StrIToA(&szTemp[StrLen(szTemp)], pDieRoll101->iBest);
        StrCat(szTemp, " of ");
        StrIToA(&szTemp[StrLen(szTemp)], pDieRoll101->iNumber);
    }
    else
    {
        StrIToA(&szTemp[StrLen(szTemp)], pDieRoll101->iNumber);
    }
    StrCat(szTemp, "d");
    StrIToA(&szTemp[StrLen(szTemp)], pDieRoll101->iDie);
    if ((pDieRoll101->iModifier > 0) && (pDieRoll101->lSuccessValue == 0))
    {
        if (pDieRoll101->iModifier > 0)
            StrCat(szTemp, "+");
        StrIToA(&szTemp[StrLen(szTemp)], pDieRoll101->iModifier);
    }
/*
	if (pDieRoll101->lSuccessValue > 0)
	{
		StrCat(szTemp, " ");
		StrCat(szTemp, g_szCompare[pDieRoll101->iSuccessCompare]);
		StrCat(szTemp, " ");
		StrIToA(&szTemp[StrLen(szTemp)], pDieRoll101->lSuccessValue);
	}
*/
	if (wLen > MAX_TEMPSTRING_LEN)
		wLen = MAX_TEMPSTRING_LEN;
	StrNCopy(charPtr, szTemp, --wLen);
	charPtr[wLen] = '\0';
}

Boolean ConvertDatabase101to110 (void)
{
	Word wTemp;
	UInt uiIndex;
	Prefs101 vPrefs101;
	Prefs110 vPrefs110;
	Results110 vResults110;
	DieRoll101Ptr pDieRoll101;
	DieRoll110Ptr pDieRoll110;
	Int i, t;

	ReadRecord(g_hRollEmDB, PREFS_RECORD, &vPrefs101, sizeof(Prefs101));

	vPrefs110.wVersion = 110;
    vPrefs110.wTotalPages = vPrefs101.wTotalPages;

	OverwriteRecord(g_hRollEmDB, PREFS_RECORD, &vPrefs110, sizeof(Prefs110));

    vResults110.wCurrentPage = vPrefs101.wCurrentPage;
    vResults110.wCurrentButton = vPrefs101.wCurrentButton;
	vResults110.lLastResult = 0;
	vResults110.lLastRolls [1] = 0;

	InsertRecord(g_hRollEmDB, RESULTS_RECORD, &vResults110, sizeof(Results110));

	wTemp = 0;
	uiIndex = SOUNDS_RECORD;
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));
	InsertRecord(g_hRollEmDB, uiIndex++, &wTemp, sizeof(Word));

	pDieRoll101 = MemPtrNew(sizeof(DieRoll101) * NUM_BUTTONS);
	pDieRoll110 = MemPtrNew(sizeof(DieRoll110) * NUM_BUTTONS);

	if (pDieRoll101 && pDieRoll110)
	{
		for (i=0; i<vPrefs110.wTotalPages; i++)
		{
			ReadRecord(g_hRollEmDB, PAGE2REC110(i), pDieRoll101, (sizeof(DieRoll101) * NUM_BUTTONS));

			// Clear the record
			MemSet(pDieRoll110, (sizeof(DieRoll110) * NUM_BUTTONS), '\0');

			for (t=0; t<NUM_BUTTONS; t++)
			{
				// Copy the record.
				StrCopy(pDieRoll110 [t].szTitle, pDieRoll101 [t].szTitle);
				pDieRoll110 [t].lField00 = pDieRoll101 [t].iDie;
				pDieRoll110 [t].lField01 = pDieRoll101 [t].iNumber;
				pDieRoll110 [t].lField02 = pDieRoll101 [t].iModifier;
				pDieRoll110 [t].lField03 = pDieRoll101 [t].iBest;
				pDieRoll110 [t].lField04 = pDieRoll101 [t].iBeepType1;
				pDieRoll110 [t].lField05 = pDieRoll101 [t].iBeepType2;
				pDieRoll110 [t].lField06 = pDieRoll101 [t].iBeepCompare1;
				pDieRoll110 [t].lField07 = pDieRoll101 [t].iBeepCompare2;
				pDieRoll110 [t].lField08 = pDieRoll101 [t].iSuccessCompare;
				pDieRoll110 [t].lField09 = pDieRoll101 [t].lBeepValue1;
				pDieRoll110 [t].lField10 = pDieRoll101 [t].lBeepValue2;
				pDieRoll110 [t].lField11 = pDieRoll101 [t].lSuccessValue;
			}

			OverwriteRecord(g_hRollEmDB, PAGE2REC110(i), pDieRoll110, (sizeof(DieRoll110) * NUM_BUTTONS));
		}
	}

    if (pDieRoll101)
        MemPtrFree(pDieRoll101);
    if (pDieRoll110)
        MemPtrFree(pDieRoll110);

	// We now have a 110 database set the version version number
	g_wVersion = 110;
}

void DrawInfo ()
{
	Char szText [128];
	Int iCurrentX, iCurrentY, iCharHeight;
    RectangleType rtRect;
	Word wVersion = 0;
	Word wTemp;

	// Open the database.
//	OpenDatabase();

	iCharHeight = FntCharHeight();

    rtRect.topLeft.x = 1;
    rtRect.topLeft.y = 17;
    rtRect.extent.x = 158;
    rtRect.extent.y = (iCharHeight * 3) + 7;

	iCurrentX = 10;
	iCurrentY = rtRect.topLeft.y;

    WinEraseRectangle(&rtRect, 0);
//	WinDrawRectangleFrame(rectangleFrame, &rtRect);

	ReadRecord(g_hRollEmDB, PREFS_RECORD, &wVersion, sizeof(Word));

	StrCopy(szText, "Roll Em Status");
	WinDrawChars(szText, StrLen(szText), iCurrentX, iCurrentY);
	iCurrentY += iCharHeight;

	iCurrentY += 5;
	StrCopy(szText, "Database:");
	WinDrawChars(szText, StrLen(szText), iCurrentX, iCurrentY);
	StrCopy(szText, (wVersion == VERSION_NUMBER) ? "Up to date" : "Out of date");
	WinDrawChars(szText, StrLen(szText), iCurrentX + 60, iCurrentY);
	iCurrentY += iCharHeight;
	if (wVersion == VERSION_NUMBER)
	{
		wTemp = 0;
		ReadRecord(g_hRollEmDB, SOUNDS_RECORD, &wTemp, sizeof(Word));

		StrCopy(szText, "Sounds:");
		WinDrawChars(szText, StrLen(szText), iCurrentX, iCurrentY);
		StrLToA(szText, (Long)wTemp);
		WinDrawChars(szText, StrLen(szText), iCurrentX + 60, iCurrentY);
		iCurrentY += iCharHeight;
	}

	// Close the database.
//	CloseDatabase();
}
