#include "StdInc.h"
#include "RollEmUpd.h"
#include "Callback.h"
#include "EditNote.h"
#include "EditTitle.h"

UChar g_szSounds [118] =
{
	0x00, 0x04, 
	0x01, 0xb8, 0x00, 0x4b, 
	0x00, 0x00, 0x00, 0x00, 
	0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x01, 0xb8, 0x00, 0x4b, 
	0x02, 0x92, 0x00, 0x4b, 
	0x00, 0x00, 0x00, 0x00, 
	0x53, 0x75, 0x63, 0x63, 0x65, 0x73, 0x73, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x02, 0x92, 0x00, 0x4b,
	0x01, 0xb8, 0x00, 0x4b,
	0x00, 0x00, 0x00, 0x00,
	0x46, 0x61, 0x69, 0x6c, 0x75, 0x72, 0x65, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x01, 0x4a, 0x00, 0x7d, 
	0x01, 0x4a, 0x00, 0x7d, 
	0x01, 0x4a, 0x00, 0x7d, 
	0x01, 0x04, 0x01, 0xf4,
	0x00, 0x00, 0x00, 0x00,
	0x35, 0x74, 0x68, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
};

SoundNode g_vSounds;
Int g_iSoundPos, g_iNotePos;
Int g_iSoundSelect, g_iNoteSelect;
Int g_iSoundRows, g_iNoteRows;

void InitializeSound ();

void LoadSoundTable ();
void LoadNoteTable ();
void LoadRows ();

Int AddNote (UInt uiSoundIndex, UInt uiNoteIndex, NotePtr pNote);
Int GetNote (UInt uiSoundIndex, UInt uiNoteIndex, NotePtr *pNote);
Int RemoveNote (UInt uiSoundIndex, UInt uiNoteIndex);
Int GetNoteCount (UInt uiSoundIndex);

Int AddSound (UInt uiSoundIndex, SoundPtr pSound);
Int GetSound (UInt uiSoundIndex, SoundPtr *pSound);
Int RemoveSound (UInt uiSoundIndex);
Int GetSoundCount ();

void ReadSounds();
void WriteSounds();

void SelectSound (Int iSound);
void SelectNote (Int iNote);

void MoveSoundList (Int iDirection);
void MoveNoteList (Int iDirection);

void RedrawSound ();
void RedrawNote ();

void TestSound ();
void DrawInfo ();

void PlayNote (UInt uiHertz, UInt uiDuration)
{
	SndCommandType sctSound;

	if ((uiHertz < 1) || (uiDuration < 1))
	{
		Long lTemp;

		lTemp = ((Long)uiDuration * (Long)SysTicksPerSecond()) / 1000;
		if (lTemp)
			SysTaskDelay(lTemp);
	}
	else
	{
		// Setup the default sound.
		sctSound.cmd = sndCmdFreqDurationAmp;
		sctSound.param3 = sndDefaultAmp;

		sctSound.param1 = uiHertz;
		sctSound.param2 = uiDuration;
		SndDoCmd((VoidPtr)0, &sctSound, false);
	}
}

// Draw routine for the sounds.
void DrawSoundRecord (VoidPtr pTable, Word wRow, Word wColumn, RectanglePtr pRect)
{
	Word wLength;
	SoundPtr pSound;
	CALLBACK_PROLOGUE   /* Work around PilotGCC 0.5.0 bug */

	FntSetFont(stdFont);

	if (GetSound((wRow + g_iSoundPos), &pSound))
	{
		wLength = StrLen(pSound->szTitle);
		WinDrawChars(pSound->szTitle, wLength, pRect->topLeft.x + 5, pRect->topLeft.y);
	}

	CALLBACK_EPILOGUE /* Work around PilotGCC 0.5.0 bug */
}

// Draw routine for notes.
void DrawNoteRecord (VoidPtr pTable, Word wRow, Word wColumn, RectanglePtr pRect)
{
	Word wLength = 0;
	Char szBuffer [32];
	NotePtr pNote;
	CALLBACK_PROLOGUE   /* Work around PilotGCC 0.5.0 bug */

	FntSetFont(stdFont);

	if (GetNote(g_iSoundSelect, (wRow + g_iNotePos), &pNote))
	{
		StrLToA(szBuffer, pNote->uiHertz);
		wLength = StrLen(szBuffer);
		WinDrawChars(szBuffer, wLength, pRect->topLeft.x + 5, pRect->topLeft.y);

		StrLToA(szBuffer, pNote->uiDuration);
		wLength = StrLen(szBuffer);
		WinDrawChars(szBuffer, wLength, pRect->topLeft.x + 55, pRect->topLeft.y);
	}

	CALLBACK_EPILOGUE /* Work around PilotGCC 0.5.0 bug */
}

Boolean SoundProc(EventPtr event)
{
    FormPtr   fpForm;
    Int       handled = 0;

	fpForm = FrmGetActiveForm();
    switch (event->eType) 
    {
        case frmOpenEvent:
        {
			Char szBuffer [32];

			InitializeSound();
			FrmDrawForm(fpForm);
			LoadSoundTable();
			RedrawSound();
			RedrawNote();
            handled = 1;
            break;
        }
        case ctlSelectEvent:  // A control button was pressed and released.
            switch (event->data.ctlEnter.controlID)
            {
			case IDC_BUTTON_SCROLL_UP_SOUND:
				MoveSoundList(-1);
				break;
			case IDC_BUTTON_SCROLL_DOWN_SOUND:
				MoveSoundList(1);
				break;
			case IDC_BUTTON_SCROLL_UP_NOTE:
				MoveNoteList(-1);
				break;
			case IDC_BUTTON_SCROLL_DOWN_NOTE:
				MoveNoteList(1);
				break;
			case IDC_BUTTON_TEST_SOUND:
				TestSound();
				break;
			case IDC_BUTTON_INSERT_SOUND:
				DoEditTitle(0);
				LoadRows();
				RedrawSound();
				break;
			case IDC_BUTTON_DELETE_SOUND:
		        if (!FrmCustomAlert(IDA_DELETE_CONFIRMATION, "sound", NULL, NULL))
		        {
					RemoveSound(g_iSoundSelect);
					LoadRows();
					RedrawSound();
					RedrawNote();
				}
				break;
			case IDC_BUTTON_EDIT_SOUND:
			{
				SoundPtr pSound;

				if (GetSound(g_iSoundSelect, &pSound))
				{
					DoEditTitle(pSound);
					RedrawSound();
				}
				break;
			}
			case IDC_BUTTON_INSERT_NOTE:
				DoEditNote(0);
				LoadRows();
				RedrawNote();
				break;
			case IDC_BUTTON_DELETE_NOTE:
		        if (!FrmCustomAlert(IDA_DELETE_CONFIRMATION, "note", NULL, NULL))
		        {
					RemoveNote(g_iSoundSelect, g_iNoteSelect);
					LoadRows();
					RedrawNote();
				}
				break;
			case IDC_BUTTON_EDIT_NOTE:
			{
				NotePtr pNote;

				if (GetNote(g_iSoundSelect, g_iNoteSelect, &pNote))
				{
					DoEditNote(pNote);
					RedrawNote();
				}
				break;
			}
            case IDC_OK:
				WriteSounds();
            case IDC_CANCEL:
				while (RemoveSound(0));
                FrmReturnToForm(0);
				DrawInfo ();
                break;
            }
            handled = 1;
            break;
		case tblSelectEvent:
		{
			switch(event->data.tblEnter.tableID)
			{
			case IDC_SOUND_TABLE:
				SelectSound(event->data.tblEnter.row);
				break;
			case IDC_NOTE_TABLE:
				SelectNote(event->data.tblEnter.row);
				break;
			default:;
			}
			break;
		}
        case menuEvent:
            switch (event->data.menu.itemID)
            {
				default:
					;
            }
            handled = 1;
            break;
    }

    return handled;
}

void InitializeSound ()
{
	static Boolean bInit = false;
	FormPtr fpForm;

	fpForm = FrmGetActiveForm();

	// Do the one time initialization 
	//  of the Sound dialog here.
	if (!bInit)
	{
		bInit = true;
	}

	// Do the every time initialization 
	//  of the Sound dialog here.
	MemSet(&g_vSounds, sizeof(SoundNode), '\0');
	ReadSounds();

	g_iSoundPos = 0;
	g_iNotePos = 0;
	g_iSoundSelect = 0;
	g_iNoteSelect = 0;
}

void LoadSoundTable ()
{
	FormPtr pForm;
	TablePtr pControl;
	Int i, iRow;

	pForm = FrmGetActiveForm();

	// Sound table.
	pControl = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, IDC_SOUND_TABLE));
	iRow = TblGetNumberOfRows(pControl);
	for (i=0; i<iRow; i++)
	{
		TblSetItemStyle(pControl, i, 0, customTableItem);
	}
	TblSetColumnUsable(pControl, 0, true);
	TblSetCustomDrawProcedure(pControl, 0, &DrawSoundRecord);

	// Note table.
	pControl = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, IDC_NOTE_TABLE));
	iRow = TblGetNumberOfRows(pControl);
	for (i=0; i<iRow; i++)
	{
		TblSetItemStyle(pControl, i, 0, customTableItem);
	}
	TblSetColumnUsable(pControl, 0, true);
	TblSetCustomDrawProcedure(pControl, 0, &DrawNoteRecord);

	LoadRows();
}

Int AddNote (UInt uiSoundIndex, UInt uiNoteIndex, NotePtr pNote)
{
	NoteNodePtr pNoteNode1, pNoteNode2;
	SoundPtr pSound;

	// Get the sound.
	if (GetSound(uiSoundIndex, &pSound) == 0)
		return 0;

	// Find the position.
	pNoteNode2 = &pSound->vNotes;
	while ((uiNoteIndex-- > 0) && (pNoteNode2->pNext))
		pNoteNode2 = (NoteNodePtr)pNoteNode2->pNext;

	// Create the note.
	pNoteNode1 = (NoteNodePtr)MemPtrNew(sizeof(NoteNode));
	if (pNote) MemMove(&(pNoteNode1->vNote), pNote, sizeof(Note));
	else MemSet(&(pNoteNode1->vNote), sizeof(Note), 0);

	// Insert it.
	pNoteNode1->pNext = (NoteNodePtr)pNoteNode2->pNext;
	pNoteNode2->pNext = (NoteNodePtr)pNoteNode1;

	// Update the note counter.
	pSound->uiNotes++;

	return 1;
}

Int GetNote (UInt uiSoundIndex, UInt uiNoteIndex, NotePtr *pNote)
{
	NoteNodePtr pNoteNode;
	SoundPtr pSound;

	// Get the sound.
	if (GetSound(uiSoundIndex, &pSound) == 0)
		return 0;

	// Find the position.
	pNoteNode = &pSound->vNotes;
	while ((uiNoteIndex-- > 0) && (pNoteNode->pNext))
		pNoteNode = (NoteNodePtr)pNoteNode->pNext;

	if (!pNoteNode->pNext)
		return 0;

	// Return the values.
	*pNote = &(((NoteNodePtr)pNoteNode->pNext)->vNote);

	return 1;
}

Int RemoveNote (UInt uiSoundIndex, UInt uiNoteIndex)
{
	NoteNodePtr pNoteNode1, pNoteNode2;
	SoundPtr pSound;

	// Get the sound.
	if (GetSound(uiSoundIndex, &pSound) == 0)
		return 0;

	// Find the position.
	pNoteNode2 = &pSound->vNotes;
	while ((uiNoteIndex-- > 0) && (pNoteNode2->pNext))
		pNoteNode2 = (NoteNodePtr)pNoteNode2->pNext;

	if (!pNoteNode2->pNext)
		return 0;

	// Remove the note.
	pNoteNode1 = (NoteNodePtr)pNoteNode2->pNext;
	pNoteNode2->pNext = pNoteNode1->pNext;
	MemPtrFree(pNoteNode1);

	// Update the note counter.
	pSound->uiNotes--;

	return 1;
}

Int GetNoteCount (UInt uiSoundIndex)
{
	NoteNodePtr pNoteNode1, pNoteNode2;
	SoundPtr pSound;

	// Get the sound.
	if (GetSound(uiSoundIndex, &pSound) == 0)
		return 0;

	return pSound->uiNotes;
}

Int AddSound (UInt uiSoundIndex, SoundPtr pSound)
{
	SoundNodePtr pSoundNode1, pSoundNode2;

	// Find the position.
	pSoundNode2 = &g_vSounds;
	while ((uiSoundIndex-- > 0) && (pSoundNode2->pNext))
		pSoundNode2 = (SoundNodePtr)pSoundNode2->pNext;

	// Create the note.
	pSoundNode1 = (SoundNodePtr)MemPtrNew(sizeof(SoundNode));
	if (pSound) MemMove(&(pSoundNode1->vSound), pSound, sizeof(Sound));
	else MemSet(&(pSoundNode1->vSound), sizeof(Sound), 0);

	// Insert it.
	pSoundNode1->pNext = (NoteNodePtr)pSoundNode2->pNext;
	pSoundNode2->pNext = (NoteNodePtr)pSoundNode1;

	return 1;
}

Int GetSound (UInt uiSoundIndex, SoundPtr *pSound)
{
	SoundNodePtr pSoundNode;

	// Find the position.
	pSoundNode = &g_vSounds;
	while ((uiSoundIndex-- > 0) && (pSoundNode->pNext))
		pSoundNode = (SoundNodePtr)pSoundNode->pNext;

	if (!pSoundNode->pNext)
		return 0;

	// Return the values.
	*pSound = &(((SoundNodePtr)pSoundNode->pNext)->vSound);

	return 1;
}

Int RemoveSound (UInt uiSoundIndex)
{
	SoundNodePtr pSoundNode1, pSoundNode2;
	NoteNodePtr pNoteNode1, pNoteNode2;

	// Find the position.
	pSoundNode2 = &g_vSounds;
	while ((uiSoundIndex-- > 0) && (pSoundNode2->pNext))
		pSoundNode2 = (SoundNodePtr)pSoundNode2->pNext;

	if (!pSoundNode2->pNext)
		return 0;

	// Remove the note.
	pSoundNode1 = (SoundNodePtr)pSoundNode2->pNext;
	pSoundNode2->pNext = pSoundNode1->pNext;

	// Remove the notes if any
	pNoteNode1 = (NoteNodePtr)pSoundNode1->vSound.vNotes.pNext;
	while (pNoteNode1)
	{
		pNoteNode2 = (NoteNodePtr)pNoteNode1->pNext;
		MemPtrFree(pNoteNode1);
		pNoteNode1 = pNoteNode2;
	}

	// Free up the sound.
	MemPtrFree(pSoundNode1);

	return 1;
}

Int GetSoundCount ()
{
	SoundNodePtr pSoundNode1, pSoundNode2;
	NoteNodePtr pNoteNode1, pNoteNode2;
	Int iCount;

	iCount = 0;

	// Find the position.
	pSoundNode2 = &g_vSounds;
	while (pSoundNode2->pNext)
	{
		pSoundNode2 = (SoundNodePtr)pSoundNode2->pNext;
		iCount++;
	}

	return iCount;
}

void ReadSounds ()
{
	VoidHand hRecord;
	Ptr pRecord;
	CharPtr pChar;
	Int i, iCount, iNotes;
	SoundPtr pSound;

	// Remove the old sounds.
	while (RemoveSound(0));

	iCount = 0;
	if (g_hRollEmDB)
	{
		if (DmNumRecords(g_hRollEmDB) > SOUNDS_RECORD)
		{
			hRecord = DmGetRecord(g_hRollEmDB, SOUNDS_RECORD);
			pChar = (CharPtr)MemHandleLock(hRecord);
			iCount = *((UIntPtr)pChar);

			// If empty default it.
			if (iCount == 0)
			{
				pChar = (CharPtr)g_szSounds;
				iCount = *((UIntPtr)pChar);
			}

			// Moveit 
			pChar += sizeof(UInt);
			for (i=0; i<iCount; i++)
			{
				AddSound(i, 0);
				GetSound(i, &pSound);

				// Add the notes.
				iNotes = 0;
				while (((NotePtr)pChar)->uiDuration)
				{
					AddNote(i, iNotes++, (NotePtr)pChar);
					pChar += sizeof(Note);
				}

				// Move up.
				pChar += sizeof(Note);
	
				// Copy the title.
				StrCopy(pSound->szTitle, pChar);

				// Skip the titles.
				pChar += MAX_TITLE_LEN + 1;
			}

			MemHandleUnlock(hRecord); 
			DmReleaseRecord(g_hRollEmDB, SOUNDS_RECORD, false);  
		}
	}
}

void WriteSounds()
{
	CharPtr pChar, pTempChar;
    NotePtr pNote;
	SoundPtr pSound;
	Note vNote;
	Int i, iSize, iSounds, iNotes, iUntitled;

	if (g_hRollEmDB)
	{
		// Count the size needed.
		iUntitled = 1;
		iSounds = 0;
		iSize = sizeof(UInt);

		while (GetSound(iSounds, &pSound) != 0)
		{
			Int iStrLen;

			iStrLen = StrLen(pSound->szTitle);
			while ((iStrLen > 0) && (pSound->szTitle [iStrLen-1] == ' '))
				iStrLen--;

			pSound->szTitle [iStrLen] = '\0';
			if (iStrLen == 0)
			{
				StrCopy(pSound->szTitle, "Untitled ");
				StrLToA((pSound->szTitle + 9), (Long)iUntitled++);
			}

			iSize += MAX_TITLE_LEN + 1;
			iSize += (pSound->uiNotes + 1) * sizeof(Note);
			iSounds++;
		}

		// Initialize.
		MemSet(&vNote, sizeof(Note), 0);

		// Load the sounds into memory.
		pChar = (CharPtr)MemPtrNew(iSize);
		MemMove(pChar, &iSounds, sizeof(UInt));
		pTempChar = pChar + sizeof(UInt);
		for (i=0; i<iSounds; i++)
		{
			// Add the notes.
			iNotes = 0;
			while (GetNote(i, iNotes++, &pNote))
			{
				MemMove(pTempChar, pNote, sizeof(Note));
				pTempChar += sizeof(Note);
			}

			// Add the terminator..
			MemMove(pTempChar, &vNote, sizeof(Note));
			pTempChar += sizeof(Note);

			// Write the title.
			GetSound(i, &pSound);
			MemMove(pTempChar, pSound->szTitle, MAX_TITLE_LEN + 1);
			pTempChar += MAX_TITLE_LEN + 1;
		}

		// Write the new record.
		OverwriteRecord(g_hRollEmDB, SOUNDS_RECORD, pChar, iSize);
		MemPtrFree(pChar);
	}
}

void SelectSound (Int iSound)
{
	g_iSoundSelect = iSound + g_iSoundPos;
	g_iNotePos = 0;
	g_iNoteSelect = 0;
	LoadRows();
	RedrawNote();
}

void SelectNote (Int iNote)
{
	g_iNoteSelect = iNote + g_iNotePos;
}

void RedrawSound ()
{
	FormPtr pForm;
	TablePtr pControl;
	Int iPos;

	pForm = FrmGetActiveForm();
	pControl = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, IDC_SOUND_TABLE));
	TblEraseTable(pControl);
	TblDrawTable(pControl);

	iPos = g_iSoundSelect - g_iSoundPos;
	if ((iPos >= 0) && (iPos < g_iSoundRows))
		TblSelectItem(pControl, iPos, 0);

	MoveSoundList(0);
}

void RedrawNote ()
{
	FormPtr pForm;
	TablePtr pControl;
	Int iPos;

	pForm = FrmGetActiveForm();
	pControl = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, IDC_NOTE_TABLE));
	TblEraseTable(pControl);
	TblDrawTable(pControl);

	iPos = g_iNoteSelect - g_iNotePos;
	if ((iPos >= 0) && (iPos < g_iNoteRows))
		TblSelectItem(pControl, iPos, 0);

	MoveNoteList(0);
}

void TestSound ()
{
	Int i;
	NotePtr pNote;

	i = 0;
	while (GetNote(g_iSoundSelect, i++, &pNote))
		PlayNote(pNote->uiHertz, pNote->uiDuration);
}

void MoveSoundList (Int iDirection)
{
	FormPtr fpForm;
	Word wUpIndex, wDownIndex;
	Boolean bScrollUp, bScrollDown;
	Int iNewPos;

	if (iDirection != 0)
	{
		iNewPos = g_iSoundPos + iDirection;

		if ((iNewPos + g_iSoundRows) > GetSoundCount())
			iNewPos = GetSoundCount() - g_iSoundRows;
		if (iNewPos < 0)
			iNewPos = 0;

		if (iNewPos != g_iSoundPos)
		{
			g_iSoundPos = iNewPos;
			RedrawSound();
		}
	}

	bScrollUp = (g_iSoundPos != 0);
	bScrollDown = ((g_iSoundPos + g_iSoundRows) < GetSoundCount());

    fpForm = FrmGetActiveForm();
	wUpIndex = FrmGetObjectIndex(fpForm, IDC_BUTTON_SCROLL_UP_SOUND);
	wDownIndex = FrmGetObjectIndex(fpForm, IDC_BUTTON_SCROLL_DOWN_SOUND);
	FrmUpdateScrollers(fpForm, wUpIndex, wDownIndex, bScrollUp, bScrollDown); 
}

void MoveNoteList (Int iDirection)
{
	FormPtr fpForm;
	Word wUpIndex, wDownIndex;
	Boolean bScrollUp, bScrollDown;
	Int iNewPos, i, iCount;

	iCount = GetNoteCount(g_iSoundSelect);
	if (iDirection != 0)
	{
		iNewPos = g_iNotePos + iDirection;

		if ((iNewPos + g_iNoteRows) > iCount)
			iNewPos = iCount - g_iNoteRows;
		if (iNewPos < 0)
			iNewPos = 0;

		if (iNewPos != g_iNotePos)
		{
			g_iNotePos = iNewPos;
			RedrawNote();
		}
	}

	bScrollUp = (g_iNotePos != 0);
	bScrollDown = ((g_iNotePos + g_iNoteRows) < iCount);

    fpForm = FrmGetActiveForm();
	wUpIndex = FrmGetObjectIndex(fpForm, IDC_BUTTON_SCROLL_UP_NOTE);
	wDownIndex = FrmGetObjectIndex(fpForm, IDC_BUTTON_SCROLL_DOWN_NOTE);
	FrmUpdateScrollers(fpForm, wUpIndex, wDownIndex, bScrollUp, bScrollDown); 
}

void LoadRows ()
{
	FormPtr pForm;
	TablePtr pControl;
	Int i, iCount;

	pForm = FrmGetActiveForm();

	pControl = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, IDC_SOUND_TABLE));
	g_iSoundRows = TblGetNumberOfRows(pControl);
	iCount = GetSoundCount();
	for (i=0; i<g_iSoundRows; i++)
		TblSetRowUsable(pControl, i, (i < iCount) ? true : false);

	pControl = FrmGetObjectPtr(pForm, FrmGetObjectIndex(pForm, IDC_NOTE_TABLE));
	g_iNoteRows = TblGetNumberOfRows(pControl);
	iCount = GetNoteCount(g_iSoundSelect);
	for (i=0; i<g_iNoteRows; i++)
		TblSetRowUsable(pControl, i, (i < iCount) ? true : false);
}
