#include "StdInc.h"
#include "TabCtrl.h"

// Structs 
typedef struct _tagTabData
{
	Word wItems;
	Word wStart;
	Word wSelected;
	CharPtr pItems;
} TabData;
typedef TabData *TabDataPtr;

// Defines
#define FRAME_TYPE	0x0301
#define TAB_TYPE	0x0201

CharPtr GetTabString (CharPtr pInput, Word wIndex);

void TabCtrlConstructor (FormPtr frm, Word wTabId, CharPtr pTabs)
{
	TabDataPtr pTabData;
	VoidHand hTabData;

	hTabData = MemHandleNew(sizeof(TabData));
	if (hTabData)
	{
		CharPtr pTemp;

		pTabData = (TabDataPtr)MemHandleLock(hTabData);
		pTabData->wItems = 0;
		pTabData->wStart = 0;
		pTabData->wSelected = 0;
		pTabData->pItems = (CharPtr)MemPtrNew(StrLen(pTabs) + 1);
		StrCopy(pTabData->pItems, pTabs);
		pTemp = pTabData->pItems;
		while (*pTemp)
		{
			if (*pTemp == '|')
			{
				*pTemp = '\0';
				pTabData->wItems++;
			}
			pTemp++;
		}
		MemHandleUnlock(hTabData);
		FrmSetGadgetData(frm, FrmGetObjectIndex(frm, wTabId), hTabData);
	}
}

void TabCtrlDestructor (FormPtr frm, Word wTabId)
{
	TabDataPtr pTabData;
	VoidHand hTabData;

	hTabData = FrmGetGadgetData(frm, FrmGetObjectIndex(frm, wTabId));
	pTabData = (TabDataPtr)MemHandleLock(hTabData);
	MemPtrFree(pTabData->pItems);
	MemHandleUnlock(hTabData);
	MemHandleFree(hTabData);
}

void TabCtrlDraw(FormPtr frm, Word wTabId)
{
	Word wGadgetIndex = FrmGetObjectIndex(frm, wTabId);
	RectangleType vCtrlRect, vTempRect;
	UInt iTabHeight;
	GadgetPtr pGadget;

	iTabHeight = FntLineHeight() + 2;

	FrmGetObjectBounds(frm, wGadgetIndex, &vTempRect);
	// Create the tab rectangle.
	vCtrlRect.topLeft.x = vTempRect.topLeft.x;
	vCtrlRect.topLeft.y = vTempRect.topLeft.y; 
	vCtrlRect.extent.x = vTempRect.extent.x;
	vCtrlRect.extent.y = iTabHeight;
	
	// Modify the bottom rectangle.
	vTempRect.topLeft.y += iTabHeight;
	vTempRect.extent.y -= iTabHeight;

	WinEraseRectangle(&vCtrlRect, 0);

	pGadget = FrmGetObjectPtr(frm, wGadgetIndex);
	if (pGadget->attr.usable)
	{
		RectangleType vClipRect;
		FontID vOldFont;
		TabDataPtr pTabData;
		VoidHand hTabData;
		Int i;

		hTabData = FrmGetGadgetData(frm, FrmGetObjectIndex(frm, wTabId));
		pTabData = (TabDataPtr)MemHandleLock(hTabData);

		WinGetClip(&vClipRect);
		WinDrawRectangleFrame(FRAME_TYPE, &vTempRect);

		WinSetClip(&vCtrlRect);
		vTempRect.topLeft.x = vCtrlRect.topLeft.x + 2;
		vTempRect.extent.y = vCtrlRect.extent.y;
		for (i=pTabData->wStart; i<pTabData->wItems; i++)
		{
			CharPtr pText;
			Word wSize;

			pText = GetTabString(pTabData->pItems, i);

			wSize = FntCharsWidth(pText, StrLen(pText));
			vTempRect.topLeft.y = vCtrlRect.topLeft.y + 2;
			vTempRect.extent.x = wSize + 1;
			if ((vTempRect.topLeft.x + vTempRect.extent.x) >= (vCtrlRect.topLeft.x + vCtrlRect.extent.x - 10))
				break;
			WinDrawChars(pText, StrLen(pText), vTempRect.topLeft.x + 1, vTempRect.topLeft.y - 1);
			if (pTabData->wSelected == i)
			{
				vTempRect.topLeft.y -= 1;
				WinEraseLine(
					vTempRect.topLeft.x,
					vCtrlRect.topLeft.y + vTempRect.extent.y - 1,
					vTempRect.topLeft.x + vTempRect.extent.x - 1, 
					vCtrlRect.topLeft.y + vTempRect.extent.y - 1);
			}
			WinDrawRectangleFrame(TAB_TYPE, &vTempRect);

			vTempRect.topLeft.x += vTempRect.extent.x + 1;
		}

		// Not tabs are shown.
		vOldFont = FntSetFont(3);
		if (pTabData->wStart != 0)
		{
			Char cTemp;

			vTempRect.topLeft.x = vCtrlRect.topLeft.x + vCtrlRect.extent.x - 10;
			vTempRect.topLeft.y = vCtrlRect.topLeft.y + 3;
			vTempRect.extent.x = 4;
			vTempRect.extent.y = 7;
			WinSetClip(&vTempRect);

			cTemp = '\023';
			WinDrawChars(&cTemp, 1, vTempRect.topLeft.x, vTempRect.topLeft.y - 2);
		}

		if (i < pTabData->wItems)
		{
			Char cTemp;

			vTempRect.topLeft.x = vCtrlRect.topLeft.x + vCtrlRect.extent.x - 5;
			vTempRect.topLeft.y = vCtrlRect.topLeft.y + 3;
			vTempRect.extent.x = 4;
			vTempRect.extent.y = 7;
			WinSetClip(&vTempRect);

			cTemp = '\023';
			WinDrawChars(&cTemp, 1, vTempRect.topLeft.x - 3, vTempRect.topLeft.y - 2);
		}
		FntSetFont(vOldFont);
		WinSetClip(&vClipRect);
		MemHandleUnlock(hTabData);
	}
	else
	{
		WinEraseRectangleFrame(FRAME_TYPE, &vTempRect);
	}
}

Word TabCtrlTap(FormPtr frm, Word wTabId, EventPtr pEvent)
{
	Word wGadgetIndex = FrmGetObjectIndex(frm, wTabId);
	Word wReturn = 0xffff;
	GadgetPtr pGadget;
	RectangleType vCtrlRect;
	RectangleType vTempRect;
	TabDataPtr pTabData;
	VoidHand hTabData;
	Int i;

	pGadget = FrmGetObjectPtr(frm, wGadgetIndex);
	if (!pGadget->attr.usable)
		return wReturn;

	FrmGetObjectBounds(frm, wGadgetIndex, &vCtrlRect);
	vCtrlRect.extent.y = FntLineHeight();
	vTempRect.topLeft.x = vCtrlRect.topLeft.x + 2;
	vTempRect.extent.y = vCtrlRect.extent.y;

	if (!RctPtInRectangle(pEvent->screenX, pEvent->screenY, &vCtrlRect))
		return wReturn;

	hTabData = FrmGetGadgetData(frm, FrmGetObjectIndex(frm, wTabId));
	pTabData = (TabDataPtr)MemHandleLock(hTabData);

	for (i=pTabData->wStart; i<pTabData->wItems; i++)
	{
		CharPtr pText;
		Word wSize;

		pText = GetTabString(pTabData->pItems, i);
		wSize = FntCharsWidth(pText, StrLen(pText));
		vTempRect.topLeft.y = vCtrlRect.topLeft.y + 2;
		vTempRect.extent.x = wSize + 1;
		if ((vTempRect.topLeft.x + vTempRect.extent.x) >= (vCtrlRect.topLeft.x + vCtrlRect.extent.x - 10))
			break;
		if (RctPtInRectangle(pEvent->screenX, pEvent->screenY, &vTempRect))
		{
			pTabData->wSelected = i;
			wReturn = pTabData->wSelected;
			TabCtrlDraw(frm, wTabId);
			MemHandleUnlock(hTabData);
			SndPlaySystemSound(sndClick);
			return wReturn;
		}
		vTempRect.topLeft.x += vTempRect.extent.x + 1;
	}

	if (pTabData->wStart != 0)
	{
		vTempRect.topLeft.x = vCtrlRect.topLeft.x + vCtrlRect.extent.x - 12;
		vTempRect.topLeft.y = vCtrlRect.topLeft.y;
		vTempRect.extent.x = 6;
		vTempRect.extent.y = vCtrlRect.extent.y;
		if (RctPtInRectangle(pEvent->screenX, pEvent->screenY, &vTempRect))
		{
			pTabData->wStart--;
			TabCtrlDraw(frm, wTabId);
			MemHandleUnlock(hTabData);
			SndPlaySystemSound(sndClick);
			return wReturn;
		}
	}

	if (i < pTabData->wItems)
	{
		vTempRect.topLeft.x = vCtrlRect.topLeft.x + vCtrlRect.extent.x - 5;
		vTempRect.topLeft.y = vCtrlRect.topLeft.y;
		vTempRect.extent.x = 6;
		vTempRect.extent.y = vCtrlRect.extent.y;
		if (RctPtInRectangle(pEvent->screenX, pEvent->screenY, &vTempRect))
		{
			pTabData->wStart++;
			TabCtrlDraw(frm, wTabId);
			MemHandleUnlock(hTabData);
			SndPlaySystemSound(sndClick);
			return wReturn;
		}
	}
	MemHandleUnlock(hTabData);

	return wReturn;
}

Word TabCtrlTotal (FormPtr frm, Word wTabId)
{
	TabDataPtr pTabData;
	VoidHand hTabData;
	Word wCount;

	hTabData = FrmGetGadgetData(frm, FrmGetObjectIndex(frm, wTabId));
	pTabData = (TabDataPtr)MemHandleLock(hTabData);
	wCount = pTabData->wItems;
	MemHandleUnlock(hTabData);

	return wCount;
}

Word TabCtrlCurrent (FormPtr frm, Word wTabId)
{
	TabDataPtr pTabData;
	VoidHand hTabData;
	Word wCount;

	hTabData = FrmGetGadgetData(frm, FrmGetObjectIndex(frm, wTabId));
	pTabData = (TabDataPtr)MemHandleLock(hTabData);
	wCount = pTabData->wSelected;
	MemHandleUnlock(hTabData);

	return wCount;
}

CharPtr GetTabString (CharPtr pInput, Word wIndex)
{
	CharPtr pTemp = pInput;

	while (wIndex-- && *pTemp)
	{
		pTemp += StrLen(pTemp) + 1;
	}

	return pTemp;
}
